part of masamune;

class LoadingBuilder extends StatefulWidget {
  const LoadingBuilder({
    required this.futures,
    required this.builder,
    this.indicatorColor,
    this.loading,
  });

  final List<FutureOr<dynamic>> futures;

  final Widget Function(BuildContext context) builder;

  /// Builder when the data is empty.
  final Widget? loading;

  /// Loading indicator color.
  final Color? indicatorColor;

  @override
  State<StatefulWidget> createState() => _LoadingBuilder();
}

class _LoadingBuilder extends State<LoadingBuilder> {
  Future<dynamic>? _wait;

  @override
  void initState() {
    super.initState();
    final futures = widget.futures.whereType<Future>();
    if (futures.isNotEmpty) {
      _wait = Future.wait(futures);
    } else {
      _wait = null;
    }
  }

  @override
  void didUpdateWidget(LoadingBuilder oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.futures != oldWidget.futures) {
      final futures = widget.futures.whereType<Future>();
      if (futures.isNotEmpty) {
        _wait = Future.wait(futures);
      } else {
        _wait = null;
      }
      setState(() {});
    }
  }

  @override
  Widget build(BuildContext context) {
    if (_wait == null) {
      return widget.builder(context);
    }
    return FutureBuilder(
      future: _wait,
      builder: (context, snapshot) {
        if (snapshot.connectionState != ConnectionState.done) {
          return Center(
            child: widget.loading ??
                context.widgetTheme.loadingIndicator?.call(
                  context,
                  widget.indicatorColor ?? context.theme.disabledColor,
                ) ??
                LoadingBouncingGrid.square(
                  backgroundColor:
                      widget.indicatorColor ?? context.theme.disabledColor,
                ),
          );
        } else {
          return widget.builder(context);
        }
      },
    );
  }
}
